@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2015-2018  Brazil
-  Copyright(C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2015-2018  Brazil
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -37,19 +37,31 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_temporary(grn_ctx *ctx, grn_obj *obj);
 
-GRN_API grn_bool
+/**
+ * \brief Check whether a Groonga built-in object is or not.
+ *
+ * \param ctx The context object.
+ * \param obj The target object to check.
+ *
+ * \return `true` if the object is a built-in Groonga object, `false` otherwise.
+ */
+GRN_API bool
 grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_bulk(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_text_family_bulk(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
+grn_obj_maybe_record_bulk(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
 grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_text_family_vector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
+grn_obj_is_number_family_vector(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
 grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_uvector(grn_ctx *ctx, grn_obj *obj);
@@ -57,7 +69,7 @@ GRN_API bool
 grn_obj_is_weight_uvector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_db(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_table(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj);
@@ -73,33 +85,41 @@ GRN_API bool
 grn_obj_is_patricia_trie(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_result_set(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_number_family_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_text_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+/**
+ * \brief Check whether the specified object is a vector column or not.
+ *
+ * \param ctx The context object.
+ * \param obj The target object to check.
+ *
+ * \return `true` if the object is a vector column, `false` otherwise.
+ */
+GRN_API bool
 grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_text_family_vector_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_data_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_index_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_accessor(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_id_accessor(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_key_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_value_accessor(grn_ctx *ctx, grn_obj *obj);
@@ -122,32 +142,51 @@ grn_obj_is_mean_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
+grn_obj_is_scalar_accessor(grn_ctx *ctx, grn_obj *obj);
+/**
+ * \brief Check whether a text family scalar accessor or not.
+ *
+ * \param ctx The context object.
+ * \param obj The target object to check.
+ *
+ * \return `true` if the object is an accessor of \ref
+ *         GRN_DB_SHORT_TEXT range table or \ref GRN_DB_SHORT_TEXT /
+ *         \ref GRN_DB_TEXT / \ref GRN_DB_LONG_TEXT scalar column,
+ *         `false` otherwise.
+ */
+GRN_API bool
+grn_obj_is_text_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
 grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
+grn_obj_is_vector_accessor(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
 grn_obj_is_type(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_text_family_type(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_tokenizer_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_function_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_selector_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_selector_only_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
+grn_obj_is_applier_proc(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
 grn_obj_is_normalizer_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_token_filter_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_scorer_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_aggregator_proc(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_visible(grn_ctx *ctx, grn_obj *obj);
@@ -157,6 +196,8 @@ GRN_API bool
 grn_obj_have_source(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool
+grn_obj_is_generated_column(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API grn_rc
 grn_obj_reindex(grn_ctx *ctx, grn_obj *obj);
@@ -165,7 +206,7 @@ GRN_API void
 grn_obj_touch(grn_ctx *ctx, grn_obj *obj, grn_timeval *tv);
 GRN_API uint32_t
 grn_obj_get_last_modified(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool
+GRN_API bool
 grn_obj_is_dirty(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API const char *
@@ -173,11 +214,18 @@ grn_obj_set_flag_to_string(int flags);
 
 GRN_API const char *
 grn_obj_type_to_string(uint8_t type);
+GRN_API bool
+grn_obj_type_is_table(uint8_t type);
+GRN_API bool
+grn_obj_type_is_column(uint8_t type);
 
-GRN_API grn_bool
+GRN_API bool
 grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len);
 
-GRN_API grn_bool
+bool
+grn_obj_is_empty(grn_ctx *ctx, grn_obj *obj, grn_id id);
+
+GRN_API bool
 grn_obj_is_corrupt(grn_ctx *ctx, grn_obj *obj);
 GRN_API size_t
 grn_obj_get_disk_usage(grn_ctx *ctx, grn_obj *obj);
