@@ -3,7 +3,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1806,7 +1807,7 @@ namespace {
 // consistent implementation but there are custom cast processes in
 // load.c and arrow.cpp for a vector value. We can reconsider the
 // current design later.
-grn_inline static grn_rc
+static inline grn_rc
 grn_caster_cast_to_record(grn_ctx *ctx, grn_caster *caster)
 {
   grn_rc rc = GRN_SUCCESS;
@@ -1911,7 +1912,7 @@ grn_caster_cast_to_record(grn_ctx *ctx, grn_caster *caster)
   return rc;
 }
 
-grn_inline static grn_rc
+static inline grn_rc
 grn_caster_cast_bool(grn_ctx *ctx, grn_caster *caster)
 {
   grn_rc rc = GRN_SUCCESS;
