@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,9 +32,13 @@ typedef struct _grn_scaner {
   uint32_t n_sis;
 } grn_scanner;
 
-grn_scanner *grn_scanner_open(grn_ctx *ctx, grn_obj *expr,
-                              grn_operator op, grn_bool record_exist);
-void grn_scanner_close(grn_ctx *ctx, grn_scanner *scanner);
+grn_scanner *
+grn_scanner_open(grn_ctx *ctx,
+                 grn_obj *expr,
+                 grn_operator op,
+                 bool record_exist);
+void
+grn_scanner_close(grn_ctx *ctx, grn_scanner *scanner);
 
 #ifdef __cplusplus
 }
