@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2009-2015  Brazil
-  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,13 +19,13 @@
 
 #include "grn_index_column.h"
 #include "grn_ii.h"
+#include "grn_float.h"
 #include "grn_hash.h"
 
 #include <string.h>
 #include <math.h>
 
 static uint64_t grn_index_sparsity = 10;
-static bool grn_index_chunk_split_enable = true;
 
 void
 grn_index_column_init_from_env(void)
@@ -43,21 +44,9 @@ grn_index_column_init_from_env(void)
       }
     }
   }
-
-  {
-    char grn_index_chunk_split_enable_env[GRN_ENV_BUFFER_SIZE];
-    grn_getenv("GRN_INDEX_CHUNK_SPLIT_ENABLE",
-               grn_index_chunk_split_enable_env,
-               GRN_ENV_BUFFER_SIZE);
-    if (strcmp(grn_index_chunk_split_enable_env, "no") == 0) {
-      grn_index_chunk_split_enable = false;
-    } else {
-      grn_index_chunk_split_enable = true;
-    }
-  }
 }
 
-grn_inline static void
+static inline void
 grn_index_column_build_call_hook(grn_ctx *ctx,
                                  grn_obj *obj,
                                  grn_id id,
@@ -192,6 +181,7 @@ grn_index_column_build(grn_ctx *ctx, grn_obj *index_column)
                      NULL,
                      NULL);
   switch (lexicon_flags & GRN_OBJ_TABLE_TYPE_MASK) {
+  case GRN_OBJ_TABLE_HASH_KEY:
   case GRN_OBJ_TABLE_PAT_KEY:
   case GRN_OBJ_TABLE_DAT_KEY:
     use_grn_ii_build = true;
@@ -225,11 +215,7 @@ grn_index_column_build(grn_ctx *ctx, grn_obj *index_column)
   }
   grn_obj_set_visibility(ctx, index_column, false);
   if (use_grn_ii_build) {
-    if (grn_index_chunk_split_enable) {
-      grn_ii_build2(ctx, ii, NULL);
-    } else {
-      grn_ii_build(ctx, ii, grn_index_sparsity);
-    }
+    grn_ii_build(ctx, ii, NULL);
   } else {
     for (i = 0; i < n_columns; i++) {
       grn_obj *column = columns[i];
@@ -264,6 +250,74 @@ grn_index_column_rebuild(grn_ctx *ctx, grn_obj *index_column)
   GRN_API_RETURN(ctx->rc);
 }
 
+static const grn_log_level grn_index_column_diff_progress_log_level_default =
+  GRN_LOG_DEBUG;
+
+grn_rc
+grn_index_column_diff_options_init(grn_ctx *ctx,
+                                   grn_index_column_diff_options *options)
+{
+  options->progress_log_level =
+    grn_index_column_diff_progress_log_level_default;
+  return GRN_SUCCESS;
+}
+
+grn_rc
+grn_index_column_diff_options_fin(grn_ctx *ctx,
+                                  grn_index_column_diff_options *options)
+{
+  return GRN_SUCCESS;
+}
+
+grn_index_column_diff_options *
+grn_index_column_diff_options_open(grn_ctx *ctx)
+{
+  GRN_API_ENTER;
+  grn_index_column_diff_options *options =
+    GRN_MALLOC(sizeof(grn_index_column_diff_options));
+  if (!options) {
+    GRN_API_RETURN(NULL);
+  }
+  grn_rc rc = grn_index_column_diff_options_init(ctx, options);
+  if (rc != GRN_SUCCESS) {
+    GRN_FREE(options);
+    GRN_API_RETURN(NULL);
+  }
+  GRN_API_RETURN(options);
+}
+
+grn_rc
+grn_index_column_diff_options_close(grn_ctx *ctx,
+                                    grn_index_column_diff_options *options)
+{
+  GRN_API_ENTER;
+  grn_rc rc = GRN_SUCCESS;
+  if (options) {
+    rc = grn_index_column_diff_options_fin(ctx, options);
+    GRN_FREE(options);
+  }
+  GRN_API_RETURN(rc);
+}
+
+grn_log_level
+grn_index_column_diff_options_get_progress_log_level(
+  grn_ctx *ctx, grn_index_column_diff_options *options)
+{
+  if (options) {
+    return options->progress_log_level;
+  } else {
+    return grn_index_column_diff_progress_log_level_default;
+  }
+}
+
+grn_rc
+grn_index_column_diff_options_set_progress_log_level(
+  grn_ctx *ctx, grn_index_column_diff_options *options, grn_log_level level)
+{
+  options->progress_log_level = level;
+  return GRN_SUCCESS;
+}
+
 typedef enum {
   GRN_INDEX_COLUMN_DIFF_TYPE_REMAINS,
   GRN_INDEX_COLUMN_DIFF_TYPE_MISSINGS,
@@ -304,6 +358,7 @@ typedef struct {
 } grn_index_column_diff_posting_list;
 
 typedef struct {
+  grn_index_column_diff_options *options;
   grn_obj *lexicon;
   grn_ii *ii;
   struct {
@@ -342,8 +397,11 @@ typedef struct {
 } grn_index_column_diff_data;
 
 static void
-grn_index_column_diff_data_init(grn_ctx *ctx, grn_index_column_diff_data *data)
+grn_index_column_diff_data_init(grn_ctx *ctx,
+                                grn_index_column_diff_data *data,
+                                grn_index_column_diff_options *options)
 {
+  data->options = options;
   GRN_PTR_INIT(&(data->source_columns), GRN_OBJ_VECTOR, GRN_ID_NIL);
   GRN_VOID_INIT(&(data->buffers.value));
   GRN_UINT32_INIT(&(data->buffers.postings), GRN_OBJ_VECTOR);
@@ -388,7 +446,8 @@ grn_index_column_diff_init_progress(grn_ctx *ctx,
   data->progress.n_records = grn_table_size(ctx, data->source_table);
   data->progress.i = 0;
   data->progress.interval = 10000;
-  data->progress.log_level = GRN_LOG_DEBUG;
+  data->progress.log_level =
+    grn_index_column_diff_options_get_progress_log_level(ctx, data->options);
   double n_records_digits = ceil(log10(data->progress.n_records + 1));
   data->progress.n_records_digits = (int)n_records_digits;
   grn_timeval_now(ctx, &(data->progress.start_time));
@@ -459,7 +518,12 @@ grn_index_column_diff_progress(grn_ctx *ctx, grn_index_column_diff_data *data)
        current_time.tv_nsec / GRN_TIME_NSEC_PER_SEC_F) -
       ((double)(previous_time->tv_sec) +
        previous_time->tv_nsec / GRN_TIME_NSEC_PER_SEC_F);
-    const double throughput = interval / current_interval_seconds;
+    double throughput;
+    if (grn_float_is_zero(current_interval_seconds)) {
+      throughput = interval;
+    } else {
+      throughput = interval / current_interval_seconds;
+    }
     const double remained_seconds =
       elapsed_seconds + ((n_records - i) / throughput);
     const char *elapsed_unit = NULL;
@@ -599,7 +663,7 @@ grn_index_column_diff_posting_list_init(
 }
 
 static void
-grn_index_column_diff_posting_list_fin(
+grn_index_column_diff_posting_list_flush(
   grn_ctx *ctx,
   grn_index_column_diff_data *data,
   grn_index_column_diff_posting_list *posting_list)
@@ -613,7 +677,6 @@ grn_index_column_diff_posting_list_fin(
                    remains,
                    GRN_BULK_HEAD(look_ahead) + offset,
                    GRN_BULK_VSIZE(look_ahead) - offset);
-    GRN_OBJ_FIN(ctx, look_ahead);
   }
 
   grn_ii_cursor *cursor = posting_list->cursor;
@@ -623,8 +686,14 @@ grn_index_column_diff_posting_list_fin(
     if (with_position) {
       while (!posting_list->need_cursor_next ||
              grn_ii_cursor_next(ctx, cursor)) {
+        if (ctx->rc != GRN_SUCCESS) {
+          return;
+        }
         grn_posting *posting;
         while ((posting = grn_ii_cursor_next_pos(ctx, cursor))) {
+          if (ctx->rc != GRN_SUCCESS) {
+            return;
+          }
           posting_list->need_cursor_next = false;
           GRN_UINT32_PUT(ctx, remains, posting->rid);
           if (with_section) {
@@ -637,14 +706,15 @@ grn_index_column_diff_posting_list_fin(
     } else {
       grn_posting *posting;
       while ((posting = grn_ii_cursor_next(ctx, cursor))) {
+        if (ctx->rc != GRN_SUCCESS) {
+          return;
+        }
         GRN_UINT32_PUT(ctx, remains, posting->rid);
         if (with_section) {
           GRN_UINT32_PUT(ctx, remains, posting->sid);
         }
       }
     }
-    grn_ii_cursor_close(ctx, cursor);
-    posting_list->cursor = NULL;
   }
 
   if (GRN_BULK_VSIZE(remains) > 0 || GRN_BULK_VSIZE(missings) > 0) {
@@ -666,8 +736,24 @@ grn_index_column_diff_posting_list_fin(
       }
     }
   }
-  GRN_OBJ_FIN(ctx, remains);
-  GRN_OBJ_FIN(ctx, missings);
+}
+
+static void
+grn_index_column_diff_posting_list_fin(
+  grn_ctx *ctx,
+  grn_index_column_diff_data *data,
+  grn_index_column_diff_posting_list *posting_list)
+{
+  if (ctx->rc == GRN_SUCCESS) {
+    grn_index_column_diff_posting_list_flush(ctx, data, posting_list);
+  }
+  GRN_OBJ_FIN(ctx, &(posting_list->remains));
+  GRN_OBJ_FIN(ctx, &(posting_list->missings));
+  GRN_OBJ_FIN(ctx, &(posting_list->look_ahead));
+  if (posting_list->cursor) {
+    grn_ii_cursor_close(ctx, posting_list->cursor);
+    posting_list->cursor = NULL;
+  }
 }
 
 static void
@@ -925,8 +1011,14 @@ grn_index_column_diff_process_token_id(grn_ctx *ctx,
     if (with_position) {
       while (!posting_list->need_cursor_next ||
              grn_ii_cursor_next(ctx, cursor)) {
+        if (ctx->rc != GRN_SUCCESS) {
+          return;
+        }
         grn_posting *posting;
         while ((posting = grn_ii_cursor_next_pos(ctx, cursor))) {
+          if (ctx->rc != GRN_SUCCESS) {
+            return;
+          }
           posting_list->need_cursor_next = false;
           const grn_index_column_diff_compared compared =
             grn_index_column_diff_compare_posting(ctx, data, posting);
@@ -977,6 +1069,9 @@ grn_index_column_diff_process_token_id(grn_ctx *ctx,
     } else {
       grn_posting *posting;
       while ((posting = grn_ii_cursor_next(ctx, cursor))) {
+        if (ctx->rc != GRN_SUCCESS) {
+          return;
+        }
         const grn_index_column_diff_compared compared =
           grn_index_column_diff_compare_posting(ctx, data, posting);
         GRN_LOG(ctx,
@@ -1238,13 +1333,16 @@ grn_index_column_diff_compute(grn_ctx *ctx, grn_index_column_diff_data *data)
 }
 
 grn_rc
-grn_index_column_diff(grn_ctx *ctx, grn_obj *index_column, grn_obj **diff)
+grn_index_column_diff_full(grn_ctx *ctx,
+                           grn_obj *index_column,
+                           grn_index_column_diff_options *options,
+                           grn_obj **diff)
 {
   grn_index_column_diff_data data = {0};
 
   GRN_API_ENTER;
 
-  grn_index_column_diff_data_init(ctx, &data);
+  grn_index_column_diff_data_init(ctx, &data, options);
   if (ctx->rc != GRN_SUCCESS) {
     GRN_API_RETURN(ctx->rc);
   }
@@ -1395,6 +1493,12 @@ exit:
   GRN_API_RETURN(ctx->rc);
 }
 
+grn_rc
+grn_index_column_diff(grn_ctx *ctx, grn_obj *index_column, grn_obj **diff)
+{
+  return grn_index_column_diff_full(ctx, index_column, NULL, diff);
+}
+
 grn_obj *
 grn_index_column_get_tokenizer(grn_ctx *ctx, grn_obj *index_column)
 {
