@@ -565,7 +565,7 @@ namespace grn {
     bool
     vectorize_batch(llama_batch &batch)
     {
-      llama_kv_cache_clear(llama_ctx_);
+      llama_kv_self_clear(llama_ctx_);
 
       if (has_encoder_ && !has_decoder_) {
         // encoder-only model
