@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2009-2017  Brazil
-  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -679,7 +680,7 @@ grn_loader_brace_set_value(grn_ctx *ctx,
   }
 }
 
-static grn_inline int
+static inline int
 name_equal(const char *p, size_t size, const char *name)
 {
   if (strlen(name) != size) {
@@ -789,7 +790,7 @@ grn_loader_parse_id_value(grn_ctx *ctx,
   }
 }
 
-static grn_inline void
+static inline void
 grn_loader_brace_add_record_internal(grn_ctx *ctx,
                                      grn_loader *loader,
                                      grn_loader_add_record_data *data)
@@ -1126,7 +1127,7 @@ grn_loader_bracket_set_values(grn_ctx *ctx,
   GRN_HASH_EACH_END(ctx, cursor);
 }
 
-static grn_inline void
+static inline void
 grn_loader_bracket_add_record_internal(grn_ctx *ctx,
                                        grn_loader *loader,
                                        grn_loader_add_record_data *data)
@@ -1436,13 +1437,13 @@ json_read(grn_ctx *ctx, grn_loader *loader, const char *str, size_t str_len)
         case 't':
           if (GRN_TEXT_LEN(loader->last) == 4 && !memcmp(v, "true", 4)) {
             loader->last->header.domain = GRN_DB_BOOL;
-            GRN_BOOL_SET(ctx, loader->last, GRN_TRUE);
+            GRN_BOOL_SET(ctx, loader->last, true);
           }
           break;
         case 'f':
           if (GRN_TEXT_LEN(loader->last) == 5 && !memcmp(v, "false", 5)) {
             loader->last->header.domain = GRN_DB_BOOL;
-            GRN_BOOL_SET(ctx, loader->last, GRN_FALSE);
+            GRN_BOOL_SET(ctx, loader->last, false);
           }
           break;
         default:
@@ -1839,20 +1840,12 @@ grn_loader_parse_columns(grn_ctx *ctx,
   return ctx->rc;
 }
 
-static grn_com_addr *addr;
-
 void
 grn_load_internal(grn_ctx *ctx, grn_load_input *input)
 {
   grn_loader *loader = &ctx->impl->loader;
 
   loader->emit_level = input->emit_level;
-  if (ctx->impl->edge) {
-    grn_edge *edge = grn_edges_add_communicator(ctx, addr);
-    grn_obj *msg = grn_msg_open(ctx, edge->com, &ctx->impl->edge->send_old);
-    /* build msg */
-    grn_edge_dispatch(ctx, edge, msg);
-  }
   if (input->table.length > 0) {
     grn_ctx_loader_clear(ctx);
     loader->input_type = input->type;
@@ -1990,9 +1983,9 @@ grn_load(grn_ctx *ctx,
     input.if_exists.length = ifexists_len;
     input.each.value = each;
     input.each.length = each_len;
-    input.output_ids = GRN_FALSE;
-    input.output_errors = GRN_FALSE;
-    input.lock_table = GRN_FALSE;
+    input.output_ids = false;
+    input.output_errors = false;
+    input.lock_table = false;
     input.emit_level = 1;
     grn_load_internal(ctx, &input);
   }
