@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1023,7 +1024,7 @@ grn_table_subrec_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
       GRN_TEXT_PUTS(ctx, buf, "section:none");
       break;
     case GRN_OBJ_UNIT_SECTION_POSITION:
-      GRN_TEXT_PUTS(ctx, buf, "section:popsition");
+      GRN_TEXT_PUTS(ctx, buf, "section:position");
       break;
     case GRN_OBJ_UNIT_POSITION_NONE:
       GRN_TEXT_PUTS(ctx, buf, "section:none");
