@@ -3,7 +3,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,7 +34,7 @@ using string_view = arrow::util::string_view;
 #endif
 
 extern "C" {
-static grn_bool grn_window_function_executor_all_tables_at_once_enable = true;
+static bool grn_window_function_executor_all_tables_at_once_enable = true;
 
 void
 grn_window_function_executor_init_from_env(void)
